@@ -4,12 +4,11 @@ import sys
 
 class ChartBuilder:
     """
-    A dynamic programming chart parameterized by semiring.
-
+    A imperative interface for specifying dynamic programs.
     """
 
     def __init__(self, score_fn=lambda a: a,
-                 semiring=ProbSemiRing,
+                 semiring=ph._LogViterbiW,
                  build_hypergraph=False,
                  debug=False):
         """
@@ -52,9 +51,9 @@ class ChartBuilder:
             self._build.__exit__(None, None, None)
             return self._hypergraph
         else:
-            return self._chart[self._last].unpack()
+            return self._chart[self._last].value
 
-    def sr(self, label):
+    def value(self, label):
         """
         Get the semiring value of the label.
 
@@ -65,7 +64,10 @@ class ChartBuilder:
         label : edge label
            Get the semiring value of the label.
         """
-        return self._semiring.make(self._scorer(label))
+        return self.sr(label)
+
+    def sr(self, label):
+        return self._semiring(self._scorer(label))
 
     def init(self, label):
         """
@@ -80,10 +82,10 @@ class ChartBuilder:
 
         if self._builder:
             node = self._build.add_node([], label=label)
-            self._chart[label] = HypergraphSemiRing([], [node], None)
+            self._chart[label] = HypergraphSemiRing(None, [], [node])
         else:
             self._chart[label] = self._semiring.one()
-        if self._debug: 
+        if self._debug:
             print >>sys.stderr, "Initing", label, label in self._chart
         return self._chart[label]
 
@@ -112,7 +114,7 @@ class ChartBuilder:
                 node = self._build.add_node(val.edges(),
                                             label=label)
                 self._chart[label] = \
-                    HypergraphSemiRing([], [node], None)
+                    HypergraphSemiRing(None, [], [node])
             else:
                 self._chart[label] = val.zero()
         else:
@@ -124,12 +126,85 @@ class ChartBuilder:
         return label in self._chart
 
     def __getitem__(self, label):
-        if self._debug: 
+        if self._debug:
             print >>sys.stderr, "Getting",label, label in self._chart
         return self._chart.get(label, self._semiring.zero())
-    
+
     def show(self):
         keys = self._chart.keys()
         keys.sort()
         for key in keys:
             print key, self._chart[key]
+
+class SemiRing(object):
+    """
+    A semiring operation.
+
+    Implements + and *
+
+    """
+    def __add__(self, other):
+        raise NotImplementedError()
+
+    def __mul__(self, other):
+        raise NotImplementedError()
+
+    @classmethod
+    def one(cls):
+        raise NotImplementedError()
+
+    @classmethod
+    def zero(cls):
+        raise NotImplementedError()
+
+    @classmethod
+    def make(cls, v):
+        raise NotImplementedError()
+
+
+class HypergraphSemiRing(SemiRing):
+    def __init__(self, edge_list=[], node_list=[],
+                 name=None, is_zero=False):
+        self.edge_list = edge_list
+        self.node_list = node_list
+        self.name = name
+        self._is_zero = is_zero
+
+    def __repr__(self):
+        return "%s %s %s %s" % (self.edge_list,
+                                self.node_list,
+                                self.name,
+                                self._is_zero)
+    def is_zero(self):
+        return self._is_zero
+
+    def __add__(self, other):
+        return HypergraphSemiRing(
+            self.edges() + other.edges())
+
+    def edges(self):
+        if self.node_list:
+            return self.edge_list + [(self.node_list, self.name)]
+        else:
+            return self.edge_list
+
+    def __mul__(self, other):
+        zero = other.is_zero() or self.is_zero()
+        if zero:
+            return HypergraphSemiRing.zero()
+        return HypergraphSemiRing([],
+                                  self.node_list + other.node_list,
+                                  other.name)
+
+    @classmethod
+    def one(cls):
+        return HypergraphSemiRing()
+
+    @classmethod
+    def zero(cls):
+        return HypergraphSemiRing(is_zero=True)
+
+    @classmethod
+    def make(cls, name):
+
+        return HypergraphSemiRing(name=name)
